@@ -5,8 +5,7 @@ import os
 import re
 
 from abc import ABC, abstractmethod
-from typing import Optional, Tuple
-from .client import _parse_temperature
+from typing import Dict, Optional, Tuple
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -15,16 +14,16 @@ def _is_write(cmd: str) -> bool:
     """Return whether cmd is a write."""
     return "!" in cmd
 
+def _parse_temperature(response: Optional[str]) -> Optional[int]:
+    if response is not None:
+        m = re.match("([0-9]+)[A-Z]", response)  # E.g., 72F
+        if m and m.group(1):
+            return int(m.group(1))
+    return None
 
-class BryantEvolutionLocalClient:
-    """
-    This class exposes methods to read and set various HVAC parameters for
-    a device connected directly to this host over a serial port.
-
-    All read methods return None on protocol errors (e.g., timeout). They
-    return "NAK" when the device responds with "NAK."
-
-    On a non-successful set_* call, the set may or may not have occurred.
+class _CoreClient:
+    """Implementation of BryantEvolutionLocalClient.
+    Differs in that it does not bind the system and zone.
     """
 
     class DevIO(ABC):
@@ -64,42 +63,50 @@ class BryantEvolutionLocalClient:
     # How long to wait for a response from the device.
     _timeout_sec = 6
 
-    def __init__(self, system_id: int, zone_id: int, device: DevIO):
+    def __init__(self, device: DevIO):
         self._device = device
-        self._system_id = system_id
-        self._zone_id = zone_id
         self._pending_reads: list[Tuple[str, asyncio.Future[str | None]]] = []
         self._pending_writes: list[Tuple[str, asyncio.Future[str | None]]] = []
         self._is_cmd_active: bool = False
 
-    async def read_current_temperature(self) -> Optional[int]:
+    async def read_current_temperature(
+        self, system_id: int, zone_id: int
+    ) -> Optional[int]:
         """Reads the current temperature."""
-        response = await self._send_command(f"S{self._system_id}Z{self._zone_id}RT?")
+        response = await self._send_command(f"S{system_id}Z{zone_id}RT?")
         return _parse_temperature(response)
 
-    async def read_cooling_setpoint(self) -> Optional[int]:
+    async def read_cooling_setpoint(
+        self, system_id: int, zone_id: int
+    ) -> Optional[int]:
         """Reads the current cooling setpoint."""
-        response = await self._send_command(f"S{self._system_id}Z{self._zone_id}CLSP?")
+        response = await self._send_command(f"S{system_id}Z{zone_id}CLSP?")
         return _parse_temperature(response)
 
-    async def set_cooling_setpoint(self, temperature: int) -> bool:
+    async def set_cooling_setpoint(
+        self, system_id: int, zone_id: int, temperature: int
+    ) -> bool:
         """Sets the cooling setpoint."""
         response = await self._send_command(
-            f"S{self._system_id}Z{self._zone_id}CLSP!{int(temperature)}"
+            f"S{system_id}Z{zone_id}CLSP!{int(temperature)}"
         )
         return response == "ACK"
 
-    async def read_heating_setpoint(self) -> Optional[int]:
+    async def read_heating_setpoint(
+        self, system_id: int, zone_id: int
+    ) -> Optional[int]:
         """Gets the heating setpoint."""
-        response = await self._send_command(f"S{self._system_id}Z{self._zone_id}HTSP?")
+        response = await self._send_command(f"S{system_id}Z{zone_id}HTSP?")
         return _parse_temperature(response)
 
-    async def read_hvac_mode(self) -> Optional[Tuple[str, bool]]:
+    async def read_hvac_mode(
+        self, system_id: int, zone_id: int
+    ) -> Optional[Tuple[str, bool]]:
         """Reads the HVAC mode (heat, cool, etc).
 
         Returns the mode and whether or not the system is active.
         """
-        response = await self._send_command(f"S{self._system_id}MODE?")
+        response = await self._send_command(f"S{system_id}MODE?")
         if not response:
             return None
 
@@ -117,31 +124,29 @@ class BryantEvolutionLocalClient:
             is_active = True
         return (mode, is_active)
 
-    async def set_heating_setpoint(self, temperature: int) -> bool:
+    async def set_heating_setpoint(
+        self, system_id: int, zone_id: int, temperature: int
+    ) -> bool:
         """Sets the heating setpoint."""
         response = await self._send_command(
-            f"S{self._system_id}Z{self._zone_id}HTSP!{int(temperature)}"
+            f"S{system_id}Z{zone_id}HTSP!{int(temperature)}"
         )
         return response == "ACK"
 
-    async def set_hvac_mode(self, hvac_mode: str) -> bool:
+    async def set_hvac_mode(self, system_id: int, zone_id: int, hvac_mode: str) -> bool:
         """Sets the HVAC mode."""
         if hvac_mode == "heat_cool":
             hvac_mode = "AUTO"
-        response = await self._send_command(
-            f"S{self._system_id}MODE!{hvac_mode.upper()}"
-        )
+        response = await self._send_command(f"S{system_id}MODE!{hvac_mode.upper()}")
         return response == "ACK"
 
-    async def read_fan_mode(self) -> Optional[str]:
+    async def read_fan_mode(self, system_id: int, zone_id: int) -> Optional[str]:
         """Reads the fan mode."""
-        return await self._send_command(f"S{self._system_id}Z{self._zone_id}FAN?")
+        return await self._send_command(f"S{system_id}Z{zone_id}FAN?")
 
-    async def set_fan_mode(self, fan_mode: str) -> bool:
+    async def set_fan_mode(self, system_id: int, zone_id: int, fan_mode: str) -> bool:
         """Sets the fan mode."""
-        response = await self._send_command(
-            f"S{self._system_id}Z{self._zone_id}FAN!{fan_mode}"
-        )
+        response = await self._send_command(f"S{system_id}Z{zone_id}FAN!{fan_mode}")
         return response == "ACK"
 
     def _check_rep(self) -> None:
@@ -172,6 +177,7 @@ class BryantEvolutionLocalClient:
         parts = result.split(":")
         if len(parts) != 2:
             _LOGGER.error("Unparseable response: %s" % result)
+            return None
         return parts[1]
 
     async def _maybe_process_commands(self) -> None:
@@ -198,3 +204,80 @@ class BryantEvolutionLocalClient:
 
         # Continue processing commands if need be.
         await self._maybe_process_commands()
+
+
+class BryantEvolutionLocalClient:
+    """
+    This class exposes methods to read and set various HVAC parameters for
+    a device connected directly to this host over a serial port.
+
+    All read methods return None on protocol errors (e.g., timeout). They
+    return "NAK" when the device responds with "NAK."
+
+    On a non-successful set_* call, the set may or may not have occurred.
+    """
+
+    _core_client_registry: Dict[str, _CoreClient] = {}
+
+    def __init__(self, system_id: int, zone_id: int, client: _CoreClient):
+        self._system_id = system_id
+        self._zone_id = zone_id
+        self._client = client
+
+    @classmethod
+    async def get_client(cls, system_id: int, zone_id: int, tty: str):
+        core_client_fut = cls._core_client_registry.get(tty, None)
+        if not core_client_fut:
+            core_client_fut = asyncio.get_running_loop().create_future()
+            cls._core_client_registry[tty] = core_client_fut
+            io = _CoreClient.ProdDevIO(tty)
+            await io.open()
+            core_client_fut.set_result(_CoreClient(io))
+        return BryantEvolutionLocalClient(system_id, zone_id, await core_client_fut)
+
+    async def read_current_temperature(self) -> Optional[int]:
+        """Reads the current temperature."""
+        return await self._client.read_current_temperature(
+            self._system_id, self._zone_id
+        )
+
+    async def read_cooling_setpoint(self) -> Optional[int]:
+        """Reads the current cooling setpoint."""
+        return await self._client.read_cooling_setpoint(self._system_id, self._zone_id)
+
+    async def set_cooling_setpoint(self, temperature: int) -> bool:
+        """Sets the cooling setpoint."""
+        return await self._client.set_cooling_setpoint(
+            self._system_id, self._zone_id, temperature
+        )
+
+    async def read_heating_setpoint(self) -> Optional[int]:
+        """Gets the heating setpoint."""
+        return await self._client.read_heating_setpoint(self._system_id, self._zone_id)
+
+    async def set_heating_setpoint(self, temperature: int) -> bool:
+        """Sets the heating setpoint."""
+        return await self._client.set_heating_setpoint(
+            self._system_id, self._zone_id, temperature
+        )
+
+    async def read_hvac_mode(self) -> Optional[Tuple[str, bool]]:
+        """Reads the HVAC mode (heat, cool, etc).
+
+        Returns the mode and whether or not the system is active.
+        """
+        return await self._client.read_hvac_mode(self._system_id, self._zone_id)
+
+    async def set_hvac_mode(self, hvac_mode: str) -> bool:
+        """Sets the HVAC mode."""
+        return await self._client.set_hvac_mode(
+            self._system_id, self._zone_id, hvac_mode
+        )
+
+    async def read_fan_mode(self) -> Optional[str]:
+        """Reads the fan mode."""
+        return await self._client.read_fan_mode(self._system_id, self._zone_id)
+
+    async def set_fan_mode(self, fan_mode: str) -> bool:
+        """Sets the fan mode."""
+        return await self._client.set_fan_mode(self._system_id, self._zone_id, fan_mode)
